
// SepaToolboxDlg.cpp: Implementierungsdatei
//
// ---------------------------------------------------------
// Beispielcode zur Verwendung der SepaTools.dll
// ---------------------------------------------------------
// SepaTools_Init				->	_LoadSepaToolsDll(...)
// SepaTools_Free				->	_UnLoadSepaToolsDll(...)
// SepaTools_GetBIC				->	OnBnClickedBtnBic()
// SepaTools_ConvertBLZKonto	->	OnBnClickedBtnIban()
// SepaTools_GetBankInfo		->	OnBnClickedBtnBankinfo()
// SepaTools_CreateXML
// SepaTools_WriteXML			->	OnBnClickedButtonXmlDatei()
// SepaTools_CloseXML
//
// Die Schnittstellendefinitionen sowie die Byteausrichtung ist in SepaToolsDll_def.h definiert
// 


#include "stdafx.h"
#include "SepaToolboxApp.h"
#include "SepaToolboxDlg.h"
#include "afxdialogex.h"

#include "SepaToolsDll_def.h"
#include <string>
#include <regex>
using namespace std;


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSepaToolboxDlg-Dialogfeld



CSepaToolboxDlg::CSepaToolboxDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CSepaToolboxDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSepaToolboxDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_KONTONUMMER, m_ctrlKtoNr);
	DDX_Control(pDX, IDC_BLZ, m_ctrlBlz);
	DDX_Control(pDX, IDC_IBAN, m_ctrlIban);
	DDX_Control(pDX, IDC_BIC, m_ctrlBic);
	DDX_Control(pDX, IDC_BANKINFO, m_ctrlBankInfo);
}

BEGIN_MESSAGE_MAP(CSepaToolboxDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BTN_IBAN, &CSepaToolboxDlg::OnBnClickedBtnIban)
	ON_BN_CLICKED(IDC_BTN_BIC, &CSepaToolboxDlg::OnBnClickedBtnBic)
	ON_BN_CLICKED(IDC_BTN_BANKINFO, &CSepaToolboxDlg::OnBnClickedBtnBankinfo)
	ON_BN_CLICKED(IDC_BUTTON_XML_DATEI, &CSepaToolboxDlg::OnBnClickedButtonXmlDatei)
END_MESSAGE_MAP()


// CSepaToolboxDlg-Meldungshandler

BOOL CSepaToolboxDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Symbol fr dieses Dialogfeld festlegen. Wird automatisch erledigt
	//  wenn das Hauptfenster der Anwendung kein Dialogfeld ist
	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden

	m_ctrlKtoNr.LimitText(10);
	m_ctrlBlz.LimitText(8);
	m_ctrlIban.LimitText(34);
	m_ctrlBic.LimitText(11);

	return TRUE;  // TRUE zurckgeben, wenn der Fokus nicht auf ein Steuerelement gesetzt wird
}

// Wenn Sie dem Dialogfeld eine Schaltflche "Minimieren" hinzufgen, bentigen Sie
//  den nachstehenden Code, um das Symbol zu zeichnen. Fr MFC-Anwendungen, die das 
//  Dokument/Ansicht-Modell verwenden, wird dies automatisch ausgefhrt.

void CSepaToolboxDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext zum Zeichnen

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Symbol in Clientrechteck zentrieren
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Symbol zeichnen
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// Die System ruft diese Funktion auf, um den Cursor abzufragen, der angezeigt wird, whrend der Benutzer
//  das minimierte Fenster mit der Maus zieht.
HCURSOR CSepaToolboxDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

bool CSepaToolboxDlg::_CheckAufbauKtoNr( const string strKtoNr ) const
{	
	// prinzipieller Aufbau, ohne Prfziffer
	const regex regExpr("[0-9]{2,10}"); 
	if(regex_match(strKtoNr, regExpr))
	{
		// die erste(n) Stellen drfen 0 sein, aber nach den nullen muss mindestens eine Stelle mit Wert enthalten sein
		const regex regExprUngleich0("([1-9]+)");  // Ziffern != 0
		smatch strMatch; // smatch == string match
		return regex_search(strKtoNr, strMatch, regExprUngleich0);
	}
	else
		return false;
}

bool CSepaToolboxDlg::_CheckAufbauBlz(const string & strBlz) const
{
	const regex regExpr("[1-9]{1}\\d{7}"); // analog zu [0-9]{8} -> Genau 8 Stellen, die erste muss > 0 sein
	return regex_match(strBlz, regExpr);
}

bool CSepaToolboxDlg::_CheckAufbauBIC(const string & szBIC) const
{
	const regex regExpr("[a-zA-Z]{6,6}[a-zA-Z2-9][a-nA-Np-zP-Z0-9]([a-zA-Z0-9]{3})?"); 
	return (regex_match(szBIC, regExpr)); // nur den prinzipiellen Aufbau prfen
}





// ---------------------------------------------------------
// Ab hier Beispiele zur SepaTools.dll
// ---------------------------------------------------------
bool CSepaToolboxDlg::_LoadSepaToolsDll(HINSTANCE &hInstLibSepaTools) const
{
	bool bInitSuccess(true);
	hInstLibSepaTools = LoadLibrary(SEPATOOLS_LIB_NAME);

	if (hInstLibSepaTools != NULL)
	{
		SepaTools_Init InitSepaTools;
		InitSepaTools = (SepaTools_Init) GetProcAddress(hInstLibSepaTools, "SepaTools_Init");
		if (InitSepaTools)
		{
			int iInitOk = InitSepaTools("",	"",	NULL, "", 0);
			// Alles ok
			if (iInitOk != 0)
				bInitSuccess=false;
		}
		else
			bInitSuccess=false;
	}
	else
		bInitSuccess=false;

	return bInitSuccess;
}


void CSepaToolboxDlg::_UnLoadSepaToolsDll(HINSTANCE hInstLibSepaTools) const
{
	SepaTools_Free FreeSepaTools;
	FreeSepaTools = (SepaTools_Free) GetProcAddress(hInstLibSepaTools, "SepaTools_Free");
	if (FreeSepaTools)
		FreeSepaTools();

	if (hInstLibSepaTools != NULL)
	{
		FreeLibrary(hInstLibSepaTools);
		hInstLibSepaTools = NULL;
	}
}


void CSepaToolboxDlg::OnBnClickedBtnIban()
{
	CString strKtoNr("");
	CString strBlz("");
	m_ctrlKtoNr.GetWindowText(strKtoNr);
	m_ctrlBlz.GetWindowText(strBlz);

	strKtoNr.Trim();
	strBlz.Trim();

	if (!_CheckAufbauKtoNr((LPCTSTR) strKtoNr))
	{
		MessageBox("Ungltige oder leere Kontonummer", "Ungltige Kontonummer");
		return;
	}
	if (!_CheckAufbauBlz((LPCTSTR) strBlz))
	{
		MessageBox("Ungltige oder leere Bankleitzahl", "Ungltige Bankleitzahl");
		return ;
	}

	CString strIBAN("");
	CString strBIC("");

	HINSTANCE hInstLibSepaTools(NULL);
	int iConvertResult(0);
	if (_LoadSepaToolsDll(hInstLibSepaTools))
	{
		SepaTools_ConvertBLZKonto ConvertBLZKonto;
		ConvertBLZKonto = (SepaTools_ConvertBLZKonto) GetProcAddress(hInstLibSepaTools, "SepaTools_ConvertBLZKonto");
		if (ConvertBLZKonto)
		{
			XMLConvertStruct oXmlConvertStruc;
			memset(&oXmlConvertStruc, 0, sizeof(XMLConvertStruct));
			strncpy_s(oXmlConvertStruc.BLZ, sizeof(oXmlConvertStruc.BLZ), (LPCTSTR) strBlz, _TRUNCATE);
			strncpy_s(oXmlConvertStruc.Konto, sizeof(oXmlConvertStruc.Konto), (LPCTSTR) strKtoNr, _TRUNCATE);
			iConvertResult = ConvertBLZKonto(&oXmlConvertStruc);
			strIBAN = oXmlConvertStruc.IBAN;
			strBIC = oXmlConvertStruc.BIC;
		}
	}
	else
	{
		MessageBox("SepaTools Dll konnte nicht geladen werden");
	}

	// hier ggfs. noch Behandlung der Rckmeldung in iConvertResult

	m_ctrlIban.SetWindowText(strIBAN);
	m_ctrlBic.SetWindowText(strBIC);

	if (hInstLibSepaTools != NULL)
		_UnLoadSepaToolsDll(hInstLibSepaTools);
}


void CSepaToolboxDlg::OnBnClickedBtnBic()
{
	CString strBlz("");
	m_ctrlBlz.GetWindowText(strBlz);

	strBlz.Trim();

	if (!_CheckAufbauBlz((LPCTSTR) strBlz))
	{
		MessageBox("Ungltige oder leere Bankleitzahl", "Ungltige Bankleitzahl");
		return ;
	}

	CString strBIC("");

	HINSTANCE hInstLibSepaTools(NULL);
	int iGetBicResult(0);
	if (_LoadSepaToolsDll(hInstLibSepaTools))
	{
		char czBic[11+1];
		SepaTools_GetBIC GetBIC;
		GetBIC = (SepaTools_GetBIC) GetProcAddress(hInstLibSepaTools, "SepaTools_GetBIC");
		if (GetBIC)
		{
			iGetBicResult = GetBIC((LPCTSTR) strBlz, czBic);

			strBIC = czBic;
		}
	}
	else
	{
		MessageBox("SepaTools Dll konnte nicht geladen werden");
	}

	// hier ggfs. noch Behandlung der Rckmeldung in iGetBicResult

	m_ctrlBic.SetWindowText(strBIC);

	if (hInstLibSepaTools != NULL)
		_UnLoadSepaToolsDll(hInstLibSepaTools);
}


void CSepaToolboxDlg::OnBnClickedBtnBankinfo()
{
	CString strBIC("");
	m_ctrlBic.GetWindowText(strBIC);

	strBIC.Trim();

	if (!_CheckAufbauBIC((LPCTSTR) strBIC))
	{
		MessageBox("Ungltige oder leere BIC", "Ungltige BIC");
		return ;
	}

	m_ctrlBankInfo.SetWindowText("");

	HINSTANCE hInstLibSepaTools(NULL);
	int iGetBicResult(0);
	
	if (_LoadSepaToolsDll(hInstLibSepaTools))
	{
		SepaTools_GetBankInfo GetBankInfo;
		GetBankInfo = (SepaTools_GetBankInfo) GetProcAddress(hInstLibSepaTools, "SepaTools_GetBankInfo");
		if (GetBankInfo)
		{
			XMLBankInfoStruct oXMLBankInfoStruct;
			memset(&oXMLBankInfoStruct, 0, sizeof(XMLBankInfoStruct));

			if (GetBankInfo((LPCTSTR) strBIC, &oXMLBankInfoStruct) == 0)
			{
				CString strAusgabe("");
				if (oXMLBankInfoStruct.Art == 0)
					strAusgabe += "Die Bank nimmt an keinem Verfahren teil.";
				else
				{
					CString strVerfahren("");
					if (oXMLBankInfoStruct.Art && 1)
						strVerfahren += "berweisungsverfahren (CT)";
					
					if (!strVerfahren.IsEmpty()) strVerfahren += ", ";

					if (oXMLBankInfoStruct.Art && 2)
						strVerfahren += "Lastschriftverfahren (DD)/(Core-Lastschrift)";

					if (!strVerfahren.IsEmpty()) strVerfahren += ", ";

					if (oXMLBankInfoStruct.Art && 4)
						strVerfahren += "B2B-Lastschrift";

					strAusgabe += strVerfahren;
				}

				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.BIC;
				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.BankName;
				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.LandKurz;
				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.Land;
				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.Strasse;
				strAusgabe += "\r\n";
				strAusgabe += oXMLBankInfoStruct.Ort;

				m_ctrlBankInfo.SetWindowText(strAusgabe);
			}
		}
	}
	else
	{
		MessageBox("SepaTools Dll konnte nicht geladen werden");
	}

	// hier ggfs. noch Behandlung der Rckmeldung in iGetBicResult

	if (hInstLibSepaTools != NULL)
		_UnLoadSepaToolsDll(hInstLibSepaTools);
}


void CSepaToolboxDlg::OnBnClickedButtonXmlDatei()
{
	HINSTANCE hInstLibSepaTools(NULL);
	
	if (_LoadSepaToolsDll(hInstLibSepaTools))
	{
		SepaTools_CreateXML CreateXML;
		CreateXML = (SepaTools_CreateXML) GetProcAddress(hInstLibSepaTools, "SepaTools_CreateXML");

		SepaTools_WriteXML WriteXML;
		WriteXML = (SepaTools_WriteXML) GetProcAddress(hInstLibSepaTools, "SepaTools_WriteXML");
		
		SepaTools_CloseXML CloseXML;
		CloseXML = (SepaTools_CloseXML) GetProcAddress(hInstLibSepaTools, "SepaTools_CloseXML");
	

		if (CreateXML && WriteXML && CloseXML)
		{
			CString szFilter("*.xml|Xml-Format (*.xml)|*.xml||");

			CFileDialog oFileDlg(FALSE, ".xml", "Test-berweisung.xml", OFN_CREATEPROMPT | OFN_LONGNAMES | OFN_NOREADONLYRETURN | OFN_OVERWRITEPROMPT, szFilter, this, 0, TRUE);
			if (oFileDlg.DoModal() == IDOK)
			{
				if (!oFileDlg.GetFileName().IsEmpty())
				{
					XMLCreateStruct oXMLCreateStruct;
					memset(&oXMLCreateStruct, 0, sizeof(XMLCreateStruct));

					strncpy_s(oXMLCreateStruct.ExportPfad, sizeof(oXMLCreateStruct.ExportPfad), (LPCTSTR) oFileDlg.GetFolderPath(), _TRUNCATE);
					oXMLCreateStruct.Lastschrift = 0; // berweisung
					strncpy_s(oXMLCreateStruct.XMLName, sizeof(oXMLCreateStruct.XMLName), (LPCTSTR) oFileDlg.GetFileName(), _TRUNCATE);
					strncpy_s(oXMLCreateStruct.EinreicherName, sizeof(oXMLCreateStruct.EinreicherName), "Mller Hans", _TRUNCATE);

					int iResultCreate = CreateXML(&oXMLCreateStruct);
					if (iResultCreate == 0)
					{
						XMLWriteStruct oXMLWriteStruct;

						// ----- Beginn erster Datensatz
						memset(&oXMLWriteStruct, 0, sizeof(XMLWriteStruct));
						strncpy_s(oXMLWriteStruct.AuftragName, sizeof(oXMLWriteStruct.AuftragName),		"Auftraggeber Max", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.AusfDatum, sizeof(oXMLWriteStruct.AusfDatum),			"23102013", _TRUNCATE);			// Bitte Datum anpassen
						strncpy_s(oXMLWriteStruct.AuftragBLZ, sizeof(oXMLWriteStruct.AuftragBLZ),		"74061813", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.Auftragkonto, sizeof(oXMLWriteStruct.Auftragkonto),	"100070998", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.EmpfName, sizeof(oXMLWriteStruct.EmpfName),			"Meierhofer Gerhard", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.EmpfBLZ, sizeof(oXMLWriteStruct.EmpfBLZ),				"74061813", _TRUNCATE);			// aus BLZ und Kontonummer wird automatisch BIC und IBAN ermittelt
						strncpy_s(oXMLWriteStruct.EmpfKonto, sizeof(oXMLWriteStruct.EmpfKonto),			"70998", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.Betrag, sizeof(oXMLWriteStruct.Betrag),				"154332", _TRUNCATE);			// Betrag in Cent, entspricht hier im Beispiel 1.543,32 
						strncpy_s(oXMLWriteStruct.Zweck1, sizeof(oXMLWriteStruct.Zweck1),				"Verwendungszweck 1", _TRUNCATE);
						strncpy_s(oXMLWriteStruct.Zweck2, sizeof(oXMLWriteStruct.Zweck2),				"Verwendungszweck 2", _TRUNCATE);

						int iResultWrite = WriteXML(&oXMLWriteStruct);
						// ----- Ende erster Datensatz                                                 // Dieser Block muss fr jeden Datensatz wiederholt werden

						XMLCloseStruct oXMLCloseStruct;
						memset(&oXMLCloseStruct, 0, sizeof(XMLCloseStruct));
						int iReturnCloseXml = CloseXML(&oXMLCloseStruct);
							
						// Hier knnen auch die Ergebnisse aus XMLCloseStruct ausgewertet werden
					}
				}
			}
		}
		// else XML Schnittstellen nicht gefunden
	}
	else
		MessageBox("SepaTools Dll konnte nicht geladen werden");

	if (hInstLibSepaTools != NULL)
		_UnLoadSepaToolsDll(hInstLibSepaTools);
}
